const path = require('path');
const Generator = require('yeoman-generator');
const fs = require('fs')
const chalk = require('chalk')

module.exports = class extends Generator {
  initializing() {
    this.props = {};
    this.mvConfig = (target,dest)=>{
      this.fs.copy(require.resolve(`../config/${target}`),this.destinationPath(dest))
    }
    this.copy = (target,dest)=>{
      this.fs.copyTpl(Array.isArray(target) ? target : this.templatePath(target), this.destinationPath(dest ? dest : target) ,{
          props: this.props
        }
      )
    }

    this.genPackage = () => {
      const commonDep = require('../config/_commonDep')
      const packageJson = require(this.destinationPath('package.json'))
      Object.assign(packageJson.dependencies,commonDep.dependencies)
      Object.assign(packageJson.devDependencies,commonDep.devDependencies)
      fs.writeFileSync(this.destinationPath('package.json'),JSON.stringify(packageJson,null,2))
    }
  }
  prompting () {
    const ques = [
      {
        name: 'name',
        message: '输入项目名称',
        default: 'simple'
      },
    ]

    return this.prompt(ques).then((ans)=>{
      Object.keys(ans).forEach((key)=>{
        this.props[key] = ans[key]
      })
      this.destinationRoot(path.join(this.props.name))
    })
  }

  configuration(){
    this.mvConfig('_editorconfig','.editorconfig')
    this.mvConfig('_gitignore','.gitignore')
    this.mvConfig('_eslintrc.js','.eslintrc.js')
    this.mvConfig('_postcss.config.js','postcss.config.js')
    this.mvConfig('_stylelintrc','.stylelintrc')
  }

  writing () {
    //copy src
    this.fs.copyTpl(
      this.templatePath('src'),
      this.destinationPath('src') ,
      {
        props: this.props
      }
    )
    //copy tools
    this.fs.copyTpl(
      this.templatePath('tools'),
      this.destinationPath('tools') ,
      {
        props: this.props
      }
    )
    this.fs.copyTpl(
      this.templatePath('package.json'),
      this.destinationPath('package.json') ,
      {
        props: this.props
      }
    )
  }

  install() {
    //generate package.json
    this.genPackage()
    this.yarnInstall();
  }

  // git() {
  //   this.spawnCommandSync('git', ['init']);
  // }
  end () {
    console.log(chalk.green.bold('\n\n🎉 Success 🎉'));
    console.log(chalk.green.bold(`
    Get started:
    - cd ${this.props.name}
    - yarn new
    - yarn start`));
  }
}
