const webpackDevServer = require('webpack-dev-server')
const webpackConfig = require('./webpack/dev.config.js')
const webpack = require('webpack')
const chalk = require('chalk')
const prepareURLs = require('./utils/prepareURLs')
const getConfig = require('./utils/getConfig')
const portfinder = require('portfinder')


async function server (pageName) {
  //获取options
  const baseConfig = getConfig(pageName)
  const devOptions = {
    compress: true,
    //代理接口
    proxy: baseConfig.proxyTable,
    overlay: true,//错误展示在浏览器
    hot: true,
    host: '0.0.0.0',//保证nginx可以代理hmr
    stats: {
      colors: true,
      logLevel: 'silent',
      warnings: false,
      chunks: false,
      children: false,
      modules: false,
    }
  }
  const options = {
    pageName,
    ...baseConfig
  }

  //启动webpack
  const config = await webpackConfig(options)
  //hot hmr 入口定义,在devOptions中必须定义host
  webpackDevServer.addDevServerEntrypoints(config,devOptions)

  const compiler = webpack(config)
  const server = new webpackDevServer(compiler,devOptions)

  let isIn = false

  const port = await portfinder.getPortPromise()
  const urls = prepareURLs(
    'http',
    '0.0.0.0',
    port
  )

  server.listen(port);

  compiler.plugin('done', stats => {
    if(isIn) return
    isIn = true

    console.log([
      `  App running at:`,
      `  - Local:   ${chalk.cyan(urls.localUrlForTerminal)}`,
      `  - Network: ${chalk.cyan(urls.lanUrlForTerminal)}`
    ].join('\n'))
    console.log()
  })
}


module.exports = server
