const path = require('./lib/path')
const webpack = require('webpack')
const UglifyJsPlugin = require('uglifyjs-webpack-plugin')
const cleanWebpack = require('clean-webpack-plugin')
const ExtractTextPlugin = require("extract-text-webpack-plugin");
const StyleExtHtmlWebpackPlugin = require('style-ext-html-webpack-plugin')
const PreloadWebpackPlugin = require('preload-webpack-plugin')
const globby = require('globby')
const { resolve , resolveLoader , getModules , getPlugins} = require('./common.config')
const getEntries = require('../utils/entry')
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;

const isReport = process.argv[4]


module.exports = async ({ pageName , title , env,dll,cdn} )=>{
  const pages = await getEntries(pageName)

  return {
    devtool: 'source-map',
    entry: pages,
    output: {
      path: path.dst,
      filename: `assets/${pageName}/js/[name].[chunkhash:8].js`,
      chunkFilename: `assets/${pageName}/js/[name].[chunkhash:8].chunk.js`,
      publicPath: cdn ? cdn : '/'
    },
    resolve: resolve(true),
    resolveLoader,
    module: getModules(pageName,true),
    plugins: getPlugins({ pageName,title , env,dll},Object.keys(pages),true).concat(
      new webpack.DefinePlugin({
        'process.env': {
          NODE_ENV: '"production"'
        }
      }),
      new ExtractTextPlugin({filename: `assets/${pageName}/css/[name].[contenthash:8].css`,allChunks: true}),
      new cleanWebpack([`dist/${pageName}`,`dist/assets/${pageName}`],{
          root : path.resolve(process.cwd(),"."),
          verbose : true,// Write logs to console.
      }),
      new webpack.optimize.ModuleConcatenationPlugin(),//Scope Hoisting 减少打包体积
      // new StyleExtHtmlWebpackPlugin(),//内敛联css,必需设置html plugin的inject为false
      new PreloadWebpackPlugin(),//资源预加载
      new UglifyJsPlugin({
        sourceMap: true,
        uglifyOptions: {
          compress: {
            warnings: false,
            drop_console: true,//删除console
          },
          mangle: {
            safari10: true
          }
        },
        parallel: true
      }),
    ).concat(isReport === 'report' ? new BundleAnalyzerPlugin() : [])
  }
}
